import importlib
import threading
import configparser
import pathlib
import pickle
import os

from ScopusWp.database import MySQLDatabaseAccess
from ScopusWp.config import PATH

# THE TEMPLATES FOR THE NON CODE FILES

CONFIG_INI_TEMPLATE = (
    '[SCOPUS]\n'
    'api-key = # YOUR API KEY'
    'url = https://api.elsevier.com/content\n'
    '\n'
    '[KITOPEN]\n'
    'url = https://publikationen.bibliothek.kit.edu/publikationslisten\n'
    '\n'
    '[WORDPRESS]\n'
    'url = # THE URL TO YOUR WP XMLRPC.PHP\n'
    'username = # YOUR WP USER\n'
    'password = # YOUR WP PASSWORD\n'
    'update_expiration = # AMOUNT OF DAYS BETWEEN UPDATES'
    '\n'
    '[MYSQL]\n'
    'host = localhost\n'
    'database = # YOUR MYSQL DATABASE NAME'
    'username = # YOUR MYSQL USER'
    'password = # YOUR PASSWORD'
    '\n'
    '[LOGGING]\n'
    'folder = logs\n'
    'debug_log_name = log\n'
    'activity_log_name = log\n'
)

AUTHORS_INI_TEMPLATE = (
    '[SHORTHAND]\n'
    'ids = [] # LIST OF SCOPUS IDS ASSOCIATED WITH AUTHOR\n'
    'first_name = # FIRST NAME\n'
    'last_name = # LAST NAME\n'
    'keywords = [] # WORDPRESS CATEGORIES TO ASSOCIATE AUTHOR PUBLICATIONS WITH\n'
    'scopus_whitelist = [] # SCOPUS AFFILIATION IDS TO ACCEPT\n'
    'scopus_blacklist = [] # SCOPUS AFFILIATION IDS TO IGNORE\n'
    '\n'
    '# You can add more authors by appending sections like the template above\n'
)

ID_JSON_TEMPLATE = (
    '{\n'
    '"counter": 0,\n'
    '"used": [],\n'
    '"unused": []\n'
    '}\n'
)

# THE SQL SCRIPTS NEEDED FOR THE SETUP

REFERENCE_SQL = (
    'CREATE TABLE reference '
    '('
    'id BIGINT PRIMARY KEY,'
    'wordpress_id BIGINT,'
    'scopus_id BIGINT,'
    'comments_updated_datetime DATETIME DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP'
    ') ENGINE INNODB;'
    'CREATE UNIQUE INDEX reference_id_uindex ON reference (id);'
    'COMMIT;'
)

COMMENT_REFERENCE_SQL = (
    'CREATE TABLE comment_reference'
    '('
    'internal_id BIGINT PRIMARY KEY,'
    'wordpress_post_id BIGINT,'
    'wordpress_comment_id BIGINT,'
    'scopus_id BIGINT'
    ') ENGINE INNODB;'
    'CREATE UNIQUE INDEX comment_reference_inernal_id_uindex ON comment_reference (internal_id);'
    'COMMIT;'
)

PUBLICATIONS_SQL = (
    'CREATE TABLE publications ('
    'scopus_id BIGINT PRIMARY KEY NOT NULL,'
    'eid VARCHAR(64),'
    'doi VARCHAR(64),'
    'creator TEXT,'
    'description LONGTEXT,'
    'journal TEXT,'
    'volume VARCHAR(64),'
    'date VARCHAR(64),'
    'authors LONGTEXT,'
    'keywords TEXT,'
    'citations TEXT'
    ') ENGINE INNODB;'
    'CREATE UNIQUE INDEX publications_scopus_id_uindex ON publications (scopus_id);'
    'CREATE UNIQUE INDEX publications_eid_uindex ON publications (eid);'
    'CREATE UNIQUE INDEX publications_doi_unidex ON publications (doi);'
    'COMMIT;'
)

PUBLICATION_CACHE_SQL = (
    'CREATE TABLE publication_cache ('
    'scopus_id BIGINT PRIMARY KEY NOT NULL,'
    'eid VARCHAR(64),'
    'doi VARCHAR(64),'
    'creator TEXT,'
    'description LONGTEXT,'
    'journal TEXT,'
    'volume VARCHAR(64),'
    'date VARCHAR(64),'
    'authors LONGTEXT,'
    'keywords TEXT,'
    'citations TEXT'
    ') ENGINE INNODB;'
    'CREATE UNIQUE INDEX publications_scopus_id_uindex ON publications (scopus_id);'
    'CREATE UNIQUE INDEX publications_eid_uindex ON publications (eid);'
    'CREATE UNIQUE INDEX publications_doi_unidex ON publications (doi);'
    'COMMIT;'
)

AUTHOR_CACHE_SQL = (
    'CREATE TABLE author_cache ('
    'author_id BIGINT PRIMARY KEY NOT NULL,'
    'first_name TEXT,'
    'last_name TEXT,'
    'h_index INT,'
    'citation_count INT,'
    'document_count INT,'
    'publications TEXT'
    ') ENGINE INNODB;'
    'CREATE UNIQUE INDEX author_cache_author_id_uindex ON author_cache (author_id);'
    'COMMIT;'
)


class ProjectPathInputController:

    def __init__(self):
        self._path = None

    @property
    def path(self):
        self.run()
        return self._path

    def run(self):
        if not self.check_config():
            self.prompt_path()

    def prompt_path(self):
        while True:
            inp = input('\nENTER PROJECT PATH: ')
            try:
                path = pathlib.Path(inp)
                if path.exists() and path.is_dir():
                    self._path = str(path)
                    break
                else:
                    print('INVALID PATH, TRY AGAIN')
                    continue
            except:
                print('INVALID INPUT, TRY AGAIN')
                continue

    def check_config(self):
        import ScopusWp.config as _config
        # In case the project path value is not empty, it is save to assume it has been set already and the path is
        # still valid
        if _config.PROJECT_PATH == '':
            print('NO PATH HAS BEEN SET')
            return False

        # Asking the user if he wants to keep that path or change it
        print('CURRENT PATH:\n{}'.format(_config.PROJECT_PATH))
        inp = input('\nDo you want to keep that path? Type "N" to change...')
        if inp.lower() == 'n' or inp.lower() == 'no':
            return False
        else:
            self._path = _config.PROJECT_PATH
            return True


class FolderSetupController:

    def __init__(self, project_path):
        self.path = project_path

        self.logs_path = pathlib.Path(self.path) / 'logs'
        self.temp_path = pathlib.Path(self.path) / 'temp'

    def run(self):
        if not self.logs_exists():
            self.create_logs_folder()
            print('CREATED LOGS FOLDER')
        else:
            print('LOGS FOLDER ALREADY EXISTS')

        if not self.temp_exists():
            self.create_temp_folder()
            print('CREATED TEMP FOLDER')
        else:
            print('TEMP FOLDER ALREADY EXISTS')

    def create_logs_folder(self):
        self.logs_path.mkdir()

    def create_temp_folder(self):
        self.temp_path.mkdir()

    def logs_exists(self):
        return self.logs_path.exists() and self.logs_path.is_dir()

    def temp_exists(self):
        return self.temp_path.exists() and self.temp_path.is_dir()


class ConfigSetupController:

    def __init__(self, project_path):
        self.project_path = project_path

        self.path = pathlib.Path(self.project_path) / 'config.ini'

    def run(self):
        if not self.exists():
            self.create()
            print('CREATED CONFIG.INI')
        else:
            print('CONFIG FILE ALREADY EXISTS. KEEPING THE EXISTING FILE')

    def create(self):
        with self.path.open(mode='w+') as file:
            file.write(CONFIG_INI_TEMPLATE)
            file.flush()

    def exists(self):
        return self.path.exists() and self.path.is_file()


class IdsJsonSetupController:

    def __init__(self, project_path):
        self.project_path = project_path

        self.path = pathlib.Path(self.project_path) / 'ids.json'

    def run(self):
        if not self.exists():
            self.create()
            print('CREATED IDS.JSON')
        else:
            print('IDS.JSON ALREADY EXISTS. KEEPING THE EXISTING FILE')

    def create(self):
        with self.path.open(mode='w+') as file:
            file.write(ID_JSON_TEMPLATE)
            file.flush()

    def exists(self):
        return self.path.exists() and self.path.is_file()


class ObservedAuthorsSetupController:

    def __init__(self, project_path):
        self.project_path = project_path

        self.path = pathlib.Path(self.project_path) / 'authors.ini'

    def run(self):
        if not self.exists():
            self.create()
            print('CREATED AUTHORS.INI')
        else:
            print('AUTHORS.INI EXISTS. KEEPING THE EXISTING FILE')

    def exists(self):
        return self.path.exists() and self.path.is_file()

    def create(self):
        with self.path.open(mode='w+') as file:
            file.write(AUTHORS_INI_TEMPLATE)


class SQLSetupController:

    def __init__(self):
        self.access = MySQLDatabaseAccess()

    def run(self):
        if not self.database_exists('publication_cache'):
            self.access.execute(PUBLICATION_CACHE_SQL)

        if not self.database_exists('publications'):
            self.access.execute(PUBLICATIONS_SQL)

        if not self.database_exists('author_cache'):
            self.access.execute(AUTHOR_CACHE_SQL)

        if not self.database_exists('reference'):
            self.access.execute(REFERENCE_SQL)

        if not self.database_exists('comment_reference'):
            self.access.execute(COMMENT_REFERENCE_SQL)

    def database_exists(self, database_name):
        try:
            sql = (
                'SELECT * FROM {database};'
            ).format(database=database_name)
            self.access.execute(sql)
            return True
        except:
            return False


class SetupController:

    def __init__(self):
        # Getting the path to the project folder
        self.path_input_controller = ProjectPathInputController()
        self.project_path = self.path_input_controller.path

        self.folder_setup_controller = FolderSetupController(self.project_path)
        self.ids_setup_controller = IdsJsonSetupController(self.project_path)
        self.config_setup_controller = ConfigSetupController(self.project_path)
        self.observed_author_setup_controller = ObservedAuthorsSetupController(self.project_path)

    def setup_files(self):
        # Setting up all the folders
        self.folder_setup_controller.run()

        # Setting up the data files
        self.config_setup_controller.run()
        self.ids_setup_controller.run()
        self.observed_author_setup_controller.run()

    def setup_database(self):
        pass

    @staticmethod
    def info():
        import ScopusWp.config as _config
        import os

        print('\nCURRENT VERSION:\n{}'.format(_config.VERSION))
        print('\nRUNNING IN FOLDER:\n{}'.format(_config.PATH))
        print('\nPACKAGE CONTENT:\n{}'.format(os.listdir(_config.PATH)))



class Publication:

    def __init__(self, id, title, description, date, doi, author_list, tag_list, journal, volume):
        self.id = id
        self.tags = tag_list
        self.title = title
        self.description = description
        self.date = date
        self.doi = doi
        self.authors = author_list
        self.journal = journal
        self.volume = volume

    @staticmethod
    def from_scopus_publication(scopus_publication, publication_id):
        author_list = []
        for author in scopus_publication.authors:
            author_tuple = (author.first_name, author.last_name)
            author_list.append(author_tuple)

        publication = Publication(
            publication_id,
            scopus_publication.title,
            scopus_publication.description,
            scopus_publication.date,
            scopus_publication.doi,
            author_list,
            scopus_publication.keywords,
            scopus_publication.journal,
            scopus_publication.volume
        )
        return publication



from wordpress_xmlrpc import Client
from wordpress_xmlrpc import WordPressPost, WordPressComment
from wordpress_xmlrpc.methods.posts import NewPost, EditPost, GetPost, DeletePost
from wordpress_xmlrpc.methods.comments import NewComment, EditComment

from ScopusWp.view import PublicationWordpressPostView
from ScopusWp.view import PublicationWordpressCommentView

import ScopusWp.config as cfg
import logging


class WordpressPublicationPostController:
    """
    This controller is in charge of actually posting the publications as wordpress posts or comments onto the website.

    """
    def __init__(self):
        # Getting the config instance for the project
        self.config = cfg.Config.get_instance()

        # Getting the logger for the the scopus part of the project
        self.logger = logging.getLogger('Wordpress')

        # Getting the url path to the xmlrpc.php file from the config file
        self.url = self.config['WORDPRESS']['url']
        # Getting the username and the password for the access to the wordpress api from the config file
        self.username = self.config['WORDPRESS']['username']
        self.password = self.config['WORDPRESS']['password']

        # Creating the client object from the login data
        self.client = Client(self.url, self.username, self.password)

    def post_publication(self, publication, keywords):
        """
        If given a generalized publication object and a list of keywords posts the publication as a new post to the
        wordpress website.

        :param publication: The generalized publication object describing  what to be posted
        :param keywords: A list of string keywords, which will be used as the categories of the post
        :return: The int wordpress id of the post
        """
        # Creating the view specifically for the wordpress posts
        post_view = PublicationWordpressPostView(publication, keywords)

        post = WordPressPost()

        post.title = post_view.get_title()
        post.excerpt = post_view.get_excerpt()
        # post.date = wp_post_view.get_date()
        post.slug = post_view.get_slug()
        post.content = post_view.get_content()
        post.date = post_view.get_date()

        post.id = self.client.call(NewPost(post))

        category_list = post_view.get_category_list()
        tag_list = post_view.get_tag_list()

        post.terms_names = {
            'category': category_list,
            'post_tag': tag_list
        }

        post.post_status = 'publish'
        post.comment_status = 'closed'

        self.client.call(EditPost(post.id, post))
        self.logger.info('WORDPRESS POSTED PUBLICATION, publication id: {}, wordpress id: {}'.format(
            publication.id,
            post.id
        ))

        return post.id

    def post_citations(self, wordpress_id, publication_list):
        """
        Given the wordpress post id of an already posted publication, this method will post the comments given by
        the list of generalized publication objects.

        :param wordpress_id: The int id of the post, which to extend with comments
        :param publication_list: A list of generalized publication objects
        :return: A list of int wordpress comment ids
        """
        comment_id_list = []
        self.enable_comments(wordpress_id)

        for publication in publication_list:
            if publication.title == '':
                continue
            comment = WordPressComment()
            comment_view = PublicationWordpressCommentView(publication)

            comment.content = comment_view.get_content()

            comment_id = self.client.call(NewComment(wordpress_id, comment))
            comment_id_list.append(comment_id)

            date_created = comment_view.get_date()
            comment.date_created = date_created
            self.client.call(EditComment(comment_id, comment))
            self.logger.info('WORDPRESS COMMENT POSTED, publication id: {}, comment id: {}'.format(
                publication.id,
                comment_id
            ))

        self.disable_comments(wordpress_id)
        return comment_id_list

    def delete_post(self, wordpress_id):
        """
        Calls a method om the wordpress site, which deletes the post of the given wordpress post id.

        :param wordpress_id: The int id of the post to delete
        :return: void
        """
        self.client.call(DeletePost(wordpress_id))

    def delete_posts(self, wordpress_id_list):
        """
        Deletes all the posts, given by the list of wordpress ids

        :param wordpress_id_list: The list of int ids
        :return: void
        """
        for wordpress_id in wordpress_id_list:
            self.delete_post(wordpress_id)

    def enable_comments(self, wordpress_id):
        """
        Calls a method on the wordpress site, which enables the comment for the post of the given wordpress id

        :param wordpress_id: The int id of the wordpress post
        :return: void
        """
        # Getting the Post
        post = self.client.call(GetPost(wordpress_id))

        # Changing the comment status of the post to open
        post.comment_status = 'open'

        self.client.call(EditPost(wordpress_id, post))

    def disable_comments(self, wordpress_id):
        """
        Calls a method on the wordpress site, which disables the comments for the post of the given wordpress post id

        :param wordpress_id: The int id of the post, whose comments to disable
        :return: void
        """
        # Getting the post
        post = self.client.call(GetPost(wordpress_id))

        # Changing the comment status
        post.comment_status = 'closed'

        self.client.call(EditPost(wordpress_id, post))



from ScopusWp.database import MySQLDatabaseAccess

from ScopusWp.scopus.data import ScopusPublication, ScopusAuthorProfile
from ScopusWp.scopus.data import from_dict, to_dict

from ScopusWp.config import PATH, Config

import json
import pathlib

import pickle
import os


###############
#   CLASSES   #
###############

# todo: Make the cache update save and the refill via delete, so progress does not get lost with connection error
# todo: in the long run, even the cache has to be a database, or anything that does not clog the RAM, maybe shelve


# TODO: Make it a dict-like object
class PublicationPersistencyInterface:

    def select(self, scopus_id):
        raise NotImplementedError()

    def select_all(self):
        raise NotImplementedError()

    def select_all_ids(self):
        raise NotImplementedError()

    def insert(self, publication):
        raise NotImplementedError()

    def contains(self, publication):
        raise NotImplementedError()

    def save(self):
        raise NotImplementedError()

    def wipe(self):
        raise NotImplementedError()


class AuthorProfilePersistencyInterface:

    def select(self, author_id):
        raise NotImplementedError()

    def select_all(self):
        raise NotImplementedError()

    def select_all_ids(self):
        raise NotImplementedError()

    def insert(self, author_profile):
        raise NotImplementedError()

    def contains(self, author):
        raise NotImplementedError()

    def save(self):
        raise NotImplementedError()

    def wipe(self):
        raise NotImplementedError()


###############
# Controllers #
###############


class ScopusBackupController:

    def __init__(self):

        self.backup_model = ScopusBackupPublicationModel()

    def insert_publication(self, publication):
        self.backup_model.insert(publication)

    def insert_multiple_publications(self, publication_list):
        for publication in publication_list:
            self.insert_publication(publication)

    def select_publication(self, scopus_id):
        return self.backup_model.select(scopus_id)

    def select_multiple_publications(self, scopus_id_list):
        publication_list = []
        for scopus_id in scopus_id_list:
            publication = self.select_publication(scopus_id)
            publication_list.append(publication)
        return publication_list

    def select_all_publications(self):
        return self.backup_model.select_all()

    def wipe(self):
        self.backup_model.wipe()

    def save(self):
        self.backup_model.save()


class ScopusCacheController:

    def __init__(self, publication_cache_model_class, author_cache_model_class):

        self.publication_cache_model = publication_cache_model_class()  # type: ScopusPublicationDatabaseCacheModel
        self.author_cache_model = author_cache_model_class()  # type: ScopusAuthorDatabaseCacheModel

    def insert_author_profile(self, author_profile):
        self.author_cache_model.insert(author_profile)

    def contains_author_profile(self, author):
        return self.author_cache_model.contains(author)

    def select_author_profile(self, author_id):
        return self.author_cache_model.select(author_id)

    def select_all_author_profiles(self):
        return self.author_cache_model.select_all()

    def insert_publication(self, publication):
        self.publication_cache_model.insert(publication)

    def insert_multiple_publications(self, publication_list):
        for publication in publication_list:
            self.insert_publication(publication)

    def contains_publication(self, publication):
        return self.publication_cache_model.contains(publication)

    def select_publication(self, scopus_id):
        return self.publication_cache_model.select(scopus_id)

    def select_multiple_publications(self, scopus_id_list):
        publication_list = []
        for scopus_id in scopus_id_list:
            publication = self.select_publication(scopus_id)
            publication_list.append(publication)
        return publication_list

    def select_all_publications(self):
        return self.publication_cache_model.select_all()

    def select_all_publication_ids(self):
        return self.publication_cache_model.select_all_ids()

    def select_all_author_ids(self):
        return self.author_cache_model.select_all_ids()

    def save(self):
        self.publication_cache_model.save()
        self.author_cache_model.save()

    def wipe(self):
        self.publication_cache_model.wipe()
        self.author_cache_model.wipe()

##########
# Models #
##########


class ScopusAuthorPickleCacheModel(AuthorProfilePersistencyInterface):

    def __init__(self):
        self.path_string = '{}/cache/authors.pkl'.format(PATH)
        self.path = pathlib.Path(self.path_string)
        self.content = self.load()

    def load(self):
        with self.path.open(mode='rb') as file:
            content = pickle.load(file)
        return content

    def insert(self, author_profile):
        self.content[int(author_profile)] = author_profile

    def select(self, author_id):
        if author_id in self.content.keys():
            return self.content[author_id]

    def select_all(self):
        author_profile_list = []
        for key in self.content.keys():
            author_profile = self.content[key]
            author_profile_list.append(author_profile)
        return author_profile_list

    def contains(self, author):
        return int(author) in self.content.keys()

    def wipe(self):
        self.content = {}
        self.save()

    def save(self):
        with self.path.open(mode="wb+") as file:
            pickle.dump(self.content, file)


class ScopusPublicationPickleCacheModel(PublicationPersistencyInterface):

    def __init__(self):

        self.path_string = '{}/cache/publications.pkl'.format(PATH)

        self.path = pathlib.Path(self.path_string)

        self.content = self.load()

    def load(self):
        with self.path.open(mode='rb') as file:
            content = pickle.load(file)
        return content

    def insert(self, publication):
        self.content[int(publication)] = publication

    def select(self, scopus_id):
        if scopus_id in self.content.keys():
            return self.content[scopus_id]

    def select_all(self):
        publication_list = []
        for key in self.content.keys():
            publication = self.content[key]
            publication_list.append(publication)
        return publication_list

    def contains(self, publication):
        return int(publication) in self.content.keys()

    def wipe(self):
        self.content = {}
        self.save()

    def save(self):
        with self.path.open(mode="wb+") as file:
            pickle.dump(self.content, file)


class ScopusAuthorDatabaseCacheModel(AuthorProfilePersistencyInterface):

    def __init__(self):
        AuthorProfilePersistencyInterface.__init__(self)

        self.database_access = MySQLDatabaseAccess()

        self.config = Config.get_instance()
        self.database_name = self.config['MYSQL']['author_cache_table']

    def wipe(self):
        sql = 'TRUNCATE {};'.format(self.database_name)
        self.database_access.execute(sql)

    def insert(self, author):
        # Converting all the data into json so it fits into the data base columns

        # Turning the list of publication scopus ids for the author into a json string
        publication_list_json = json.dumps(author.publications).replace('"', "'")

        sql = (
            'INSERT INTO '
            '{database} ('
            'author_id,'
            'first_name, '
            'last_name, '
            'h_index, '
            'citation_count, '
            'document_count, '
            'publications) '
            'VALUES ('
            '{author_id},'
            '"{first_name}",'
            '"{last_name}",'
            '{h_index},'
            '{citation_count},'
            '{document_count},'
            '"{publications}") '
            'ON DUPLICATE KEY UPDATE '
            'author_id = {author_id},'
            'first_name = "{first_name}",'
            'last_name = "{last_name}",'
            'h_index = {h_index},'
            'citation_count = {citation_count},'
            'document_count = {document_count},'
            'publications = "{publications}";'
            'COMMIT;'
        ).format(
            database=self.database_name,
            author_id=author.id,
            first_name=author.first_name,
            last_name=author.last_name,
            h_index=author.h_index,
            document_count=author.document_count,
            citation_count=author.citation_count,
            publications=publication_list_json
        )

        self.database_access.execute(sql)

    def select(self, author_id):
        sql = (
            'SELECT '
            'author_id, '
            'first_name, '
            'last_name, '
            'h_index, '
            'citation_count, '
            'document_count, '
            'publications '
            'FROM {database} WHERE author_id={author_id}'
        ).format(
            database=self.database_name,
            author_id=author_id
        )

        row_list = self.database_access.select(sql)

        if len(row_list) != 0:
            row = row_list[0]
            author_profile = self._author_profile_from_list(row)
            return author_profile

    @staticmethod
    def _author_profile_from_list(row):
        author_id = row[0]
        first_name = row[1]
        last_name = row[2]
        h_index = row[3]
        citation_count = row[4]
        document_count = row[5]

        publications_json_string = row[6]
        publication_list = json.loads(publications_json_string.replace("'", '"'))
        publication_list = list(map(lambda x: int(x), publication_list))

        author_profile = ScopusAuthorProfile(
            author_id,
            first_name,
            last_name,
            h_index,
            citation_count,
            document_count,
            publication_list
        )

        return author_profile

    def select_all(self):

        sql = (
            'SELECT '
            'author_id, '
            'first_name, '
            'last_name, '
            'h_index, '
            'citation_count, '
            'document_count, '
            'publications'
            'FROM {database}'
        ).format(
            database=self.database_name
        )

        row_list = self.database_access.select(sql)
        author_profile_list = []
        for row in row_list:
            author_profile = self._author_profile_from_list(row)
            author_profile_list.append(author_profile)

        return author_profile_list

    def select_all_ids(self):
        sql = (
            'SELECT '
            'author_id '
            'FROM {database};'
        ).format(
            database=self.database_name
        )

        row_list = self.database_access.select(sql)
        id_list = list(map(lambda x: x[0], row_list))
        return id_list

    def contains(self, author):

        sql = (
            'SELECT * FROM {database} WHERE author_id={author_id}'
        ).format(
            database=self.database_name,
            author_id=int(author)
        )

        row_list = self.database_access.select(sql)
        return len(row_list) > 0

    def save(self):
        self.database_access.save()


class ScopusPublicationDatabaseCacheModel(PublicationPersistencyInterface):

    def __init__(self):
        PublicationPersistencyInterface.__init__(self)

        self.database_access = MySQLDatabaseAccess()

        self.config = Config.get_instance()
        self.database_name = self.config['MYSQL']['publication_cache_table']

    def wipe(self):
        sql = 'TRUNCATE {};'.format(self.database_name)
        self.database_access.execute(sql)

    def insert(self, publication):
        if publication is None or publication.id == '':
            return None
        # Converting all the data so it fits into a database data column

        # Turning the creator ScopusAuthor object into a json string
        creator_dict = to_dict(publication.creator)
        creator_json_string = json.dumps(creator_dict).replace('"', "'")

        # Turning the authors list, list of ScopusAuthor objects into a json string
        author_dict_list = to_dict(publication.authors)
        authors_json_string = json.dumps(author_dict_list).replace('"', "'")

        # Turning the keywords list and the citations list of str/int into a json object
        keywords_json_string = json.dumps(publication.keywords).replace('"', "'")
        citations_json_string = json.dumps(publication.citations).replace('"', "'")

        sql = (
            'INSERT INTO '
            '{database} ('
            'scopus_id, '
            'eid, '
            'doi, '
            'creator, '
            'title, '
            'description, '
            'journal, '
            'volume, '
            'date, '
            'authors, '
            'keywords, '
            'citations ) '
            'VALUES '
            '({scopus_id}, '
            '"{eid}", '
            '"{doi}", '
            '"{creator}", '
            '"{title}", '
            '"{description}", '
            '"{journal}", '
            '"{volume}", '
            '"{date}", '
            '"{authors}", '
            '"{keywords}", '
            '"{citations}") '
            'ON DUPLICATE KEY UPDATE '
            'scopus_id = {scopus_id}, '
            'eid = "{eid}", '
            'doi = "{doi}", '
            'creator = "{creator}", '
            'title = "{title}", '
            'description = "{description}", '
            'journal = "{journal}", '
            'volume = "{volume}", '
            'date = "{date}", '
            'authors = "{authors}", '
            'keywords = "{keywords}", '
            'citations = "{citations}";'
            'COMMIT;'
        ).format(
            database=self.database_name,
            scopus_id=publication.id,
            eid=publication.eid,
            doi=publication.doi,
            creator=creator_json_string,
            title=publication.title,
            description=publication.description,
            journal=publication.journal,
            volume=publication.volume,
            date=publication.date,
            authors=authors_json_string,
            keywords=keywords_json_string,
            citations=citations_json_string
        )

        # Executing the sql on the database
        self.database_access.execute(sql)

    def select(self, scopus_id):

        sql = (
            'SELECT '
            'scopus_id, '
            'eid, '
            'doi, '
            'creator, '
            'title, '
            'description, '
            'journal, '
            'volume, '
            'date, '
            'authors, '
            'keywords, '
            'citations '
            'FROM {database} WHERE scopus_id={id}'
        ).format(
            database=self.database_name,
            id=scopus_id
        )

        row_list = self.database_access.select(sql)

        if len(row_list) != 0:
            row = row_list[0]

            scopus_id = row[0]
            eid = row[1]
            doi = row[2]
            title = row[4]
            description = row[5]
            journal = row[6]
            volume = row[7]
            date = row[8]

            creator_json_string = row[3].replace("'", '"')
            creator_dict = json.loads(creator_json_string)
            creator = from_dict(creator_dict)

            authors_json_string = row[9].replace("'", '"')
            author_dict_list = json.loads(authors_json_string)
            author_list = from_dict(author_dict_list)

            keywords_json_string = row[10].replace("'", '"')
            keywords_list = json.loads(keywords_json_string)

            citations_json_string = row[11].replace("'", '"')
            citations_list = json.loads(citations_json_string)

            publication = ScopusPublication(
                scopus_id,
                eid,
                doi,
                title,
                description,
                date,
                creator,
                author_list,
                citations_list,
                keywords_list,
                journal,
                volume
            )

            return publication

    def contains(self, publication):
        if publication == '':
            return False

        sql = (
            'SELECT * FROM {database} WHERE scopus_id={scopus_id}'
        ).format(
            database=self.database_name,
            scopus_id=int(publication)
        )

        row_list = self.database_access.select(sql)
        return len(row_list) > 0

    def select_all(self):
        scopus_id_list = self.select_all_ids()
        publication_list = []
        for scopus_id in scopus_id_list:
            publication = self.select(scopus_id)
            publication_list.append(publication)
        return publication_list

    def select_all_ids(self):
        sql = (
            'SELECT '
            'scopus_id '
            'FROM {database};'
        ).format(
            database=self.database_name
        )

        row_list = self.database_access.select(sql)
        id_list = list(map(lambda x: x[0], row_list))
        return id_list

    def save(self):
        self.database_access.save()


class ScopusBackupPublicationModel(PublicationPersistencyInterface):

    def __init__(self):

        self.database_access = MySQLDatabaseAccess()
        self.database_name = 'publications'

    def execute(self, sql):
        return self.database_access.select(sql)

    def insert(self, publication):
        if not isinstance(publication.id, int):
            return None
        # Converting all the data so it fits into a database data column

        # Turning the creator ScopusAuthor object into a json string
        creator_dict = to_dict(publication.creator)
        creator_json_string = json.dumps(creator_dict).replace('"', "'")

        # Turning the authors list, list of ScopusAuthor objects into a json string
        author_dict_list = to_dict(publication.authors)
        authors_json_string = json.dumps(author_dict_list).replace('"', "'")

        # Turning the keywords list and the citations list of str/int into a json object
        keywords_json_string = json.dumps(publication.keywords).replace('"', "'")
        citations_json_string = json.dumps(publication.citations).replace('"', "'")

        sql = (
            'INSERT INTO '
            '{database} ('
            'scopus_id, '
            'eid, '
            'doi, '
            'creator, '
            'title, '
            'description, '
            'journal, '
            'volume, '
            'date, '
            'authors, '
            'keywords, '
            'citations ) '
            'VALUES '
            '({scopus_id}, '
            '"{eid}", '
            '"{doi}", '
            '"{creator}", '
            '"{title}", '
            '"{description}", '
            '"{journal}", '
            '"{volume}", '
            '"{date}", '
            '"{authors}", '
            '"{keywords}", '
            '"{citations}") '
            'ON DUPLICATE KEY UPDATE '
            'scopus_id = {scopus_id}, '
            'eid = "{eid}", '
            'doi = "{doi}", '
            'creator = "{creator}", '
            'title = "{title}", '
            'description = "{description}", '
            'journal = "{journal}", '
            'volume = "{volume}", '
            'date = "{date}", '
            'authors = "{authors}", '
            'keywords = "{keywords}", '
            'citations = "{citations}";'
        ).format(
            database=self.database_name,
            scopus_id=publication.id,
            eid=publication.eid,
            doi=publication.doi,
            creator=creator_json_string,
            title=publication.title,
            description=publication.description,
            journal=publication.journal,
            volume=publication.volume,
            date=publication.date,
            authors=authors_json_string,
            keywords=keywords_json_string,
            citations=citations_json_string
        )

        # Executing the sql on the database
        self.database_access.execute(sql)

    def select(self, scopus_id):

        sql = (
            'SELECT '
            'scopus_id, '
            'eid, '
            'doi, '
            'creator, '
            'title, '
            'description, '
            'journal, '
            'volume, '
            'date, '
            'authors, '
            'keywords, '
            'citations '
            'FROM {database} WHERE scopus_id={id}'
        ).format(
            database=self.database_name,
            id=scopus_id
        )

        row_list = self.database_access.select(sql)

        if len(row_list) != 0:
            row = row_list[0]

            scopus_id = row[0]
            eid = row[1]
            doi = row[2]
            title = row[4]
            description = row[5]
            journal = row[6]
            volume = row[7]
            date = row[8]

            creator_json_string = row[3].replace("'", '"')
            creator_dict = json.loads(creator_json_string)
            creator = from_dict(creator_dict)

            authors_json_string = row[9].replace("'", '"')
            author_dict_list = json.loads(authors_json_string)
            author_list = from_dict(author_dict_list)

            keywords_json_string = row[10].replace("'", '"')
            keywords_list = json.loads(keywords_json_string)

            citations_json_string = row[11].replace("'", '"')
            citations_list = json.loads(citations_json_string)

            publication = ScopusPublication(
                scopus_id,
                eid,
                doi,
                title,
                description,
                date,
                creator,
                author_list,
                citations_list,
                keywords_list,
                journal,
                volume
            )

            return publication

    def save(self):
        self.database_access.save()

    def select_all(self):
        # Todo: Make this to be more efficient by combining into one query
        scopus_id_list = self.select_all_ids()
        publication_list = []
        for scopus_id in scopus_id_list:
            publication = self.select(scopus_id)
            publication_list.append(publication)
        return publication_list

    def select_all_ids(self):

        sql = (
            'SELECT scopus_id FROM publications'
        )

        scopus_id_list = []
        row_list = self.database_access.select(sql)
        for row in row_list:
            scopus_id = row[0]
            scopus_id_list.append(scopus_id)
        return scopus_id_list

    def wipe(self):
        sql = "TRUNCATE publications"
        self.execute(sql)


class TempPersistentSequenceModel:

    def __init__(self, id, folder_path, name_function=None):
        # This is the id of the model, which enables to identify all the files which belong to one model, which enables
        # the possibility of multiple models using the same folder for storage
        self.id = id

        # The path to the folder, which is supposed to contain all the files for the model
        self.path_string = folder_path

        # The ath to the info file. The info file contains the info about the current state of the counter for the model
        # of the specific id and the list of paths belonging to that model
        self.info_path_string = '{}/_info_{}.json'.format(self.path_string, self.id)

        # Stores the max amount of items in the sequence, used for creating the file names for the files which contain
        # the new objects
        self.index_counter = 0

        # The keys are the path strings to the files, which contain the binary data to the objects and the values are
        # the actual values
        self.content = {}

        if name_function is None:
            self.name_function = lambda x: str(x.__class__).replace('<class', '').replace("'", '').replace('>', '')
        else:
            self.name_function = name_function

    def load(self):
        # Loading the info file, which will create a entry for each file path in the content dict with the dummy value
        # None
        self.load_info()

        for path in self.content.keys():
            # Loading the objects
            self.load_object(path)

    def load_info(self):
        if os.path.exists(self.info_path_string):
            with open(self.info_path_string, mode='r') as file:
                info_dict = json.load(file)

            # Assigning the saved index counter
            self.index_counter = info_dict['counter']
            # Adding a dummy entry for every file path listed in the info file
            for path in info_dict['paths']:
                self.content[path] = None

    def load_object(self, path):
        with open(path, mode='rb') as file:
            obj = pickle.load(file)
            self.content[path] = obj

    def save_info(self):
        # Creating the info dict
        info_dict = {
            'counter': self.index_counter,
            'paths': list(self.content.keys())
        }

        # Actually saving that dict as a json to the file
        with open(self.info_path_string, mode='w+') as file:
            json.dump(info_dict, file)

    def save_object(self, obj, path):
        # Actually creating the file and saving the object as pickled data into it
        with open(path, mode='wb+') as file:
            pickle.dump(obj, file)

    def create_path(self, obj):
        # The file name for saving an object consists of the class name of the object saved, the index counter to show
        # which part of the sequence it is and the id of the model.
        object_file_name = self.name_function(obj)
        file_name_string = '{}_{}_{}.pkl'.format(str(self.id), str(self.index_counter), object_file_name)

        file_path_string = '{}/{}'.format(self.path_string, file_name_string)
        return file_path_string

    def append(self, obj):
        # Incrementing the counter for the saved files
        self.index_counter += 1

        # Creating the path for the file and saving the key value tuple with path and object in the internal dict
        file_path_string = self.create_path(obj)
        self.content[file_path_string] = obj

        # Saving the object into a file and saving the new info data
        self.save_object(obj, file_path_string)
        self.save_info()

    def __iter__(self):
        return self.content.values().__iter__()



from ScopusWp.controller import TopController

import optparse

# TODO: Make a fail proof for the citation comment wordpress not well formed
# TODO: Make the new citations functionality with update date
# TODO: Make update date into the main reference
# TODO: Make error safe for http request pool exceptions wrapper method for requests


def main():

    parser = optparse.OptionParser()
    (options, args) = parser.parse_args()

    if len(args) != 1:
        raise ValueError('Incorrect amount of arguments passed to ScopusWp')

    if args[0] not in ['publications', 'citations', 'wipe']:
        raise ValueError('Incorrect argument passed to Scopus Wp')

    if args[0] == 'publications':
        controller = TopController()
        try:
            controller.update_publications_website()
        finally:
            controller.close()

    elif args[0] == 'citations':
        controller = TopController()
        try:
            controller.update_publications_website()
        finally:
            controller.close()

    elif args[0] == 'wipe':
        controller = TopController()
        controller.wipe_website()
        controller.close()


if __name__ == '__main__':
    #main()
    controller = TopController()
    try:
        controller.update_publications_website()
    finally:
        controller.close()



from ScopusWp.config import PATH
from ScopusWp.config import Config

from ScopusWp.database import MySQLDatabaseAccess

from ScopusWp.data import Publication

import datetime

import json
import pathlib

DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S'


class IDManagerInterface:

    def new(self):
        raise NotImplementedError()

    def delete(self, publication_id):
        raise NotImplementedError

    @property
    def all(self):
        raise NotImplementedError()

    @property
    def used(self):
        raise NotImplementedError()

    @property
    def unused(self):
        raise NotImplementedError()


class IDManagerSingleton(IDManagerInterface):

    _instance = None

    def __init__(self):
        self.path_string = '{}/ids.json'.format(PATH)
        self.path = pathlib.Path(self.path_string)

        content_dict = self.load()
        self.used_ids = content_dict['used']
        self.unused_ids = content_dict['unused']
        self.pointer = content_dict['pointer']

    def save(self):
        """
        Saves the the content dict, which contains the info about the used ids into a json file in the project folder

        :return: void
        """
        with self.path.open(mode='w') as file:
            content_dict = {
                'used': self.used_ids,
                'unused': self.unused_ids,
                'pointer': self.pointer
            }
            json.dump(content_dict, file)

    def new(self):
        """
        Creates and returns a new id to use.

        Updates the created id into the list of used ids directly and increments the pointer, that indicates the amount
        of used ids.
        :return: The int id that has been created
        """
        self.pointer += 1
        self.used_ids.append(self.pointer)
        return self.pointer

    def delete(self, publication_id):
        if publication_id in self.used_ids:
            self.used_ids.remove(publication_id)
            self.unused_ids.append(publication_id)

    def load(self):
        """
        Loads the information about the used ids from the json file in the project folder.

        :return: the dict, that contains the list if used and unused ids as well as the pointer for the amount of ids
        """
        with self.path.open(mode='r') as file:
            content_dict = json.load(file)
        return content_dict

    @staticmethod
    def get_instance():
        if IDManagerSingleton._instance is None:
            # Creating a new instance
            id_manager = IDManagerSingleton()
            IDManagerSingleton._instance = id_manager
            return id_manager
        else:
            return IDManagerSingleton._instance

    @property
    def all(self):
        return self.used_ids

    @property
    def used(self):
        return self.used_ids

    @property
    def unused(self):
        return self.unused_ids


class ReferenceController:

    def __init__(self):

        self.reference_model = PostReferenceModel()

        self.id_manager = IDManagerSingleton.get_instance()

        # The model for the comment reference model
        self.comment_reference_model = CommentReferenceModel()

    def select_reference(self, internal_id):
        return self.reference_model.select(internal_id)

    def select_all_references(self):
        """
        Returns a list of tuples, where each tuple represents one entry in the reference database, with the first
        item being the internal id for the publication, the second being the wordpress id and the third being the
        scopus id for the publication.

        :return: A list of tuples with three items each
        """
        return self.reference_model.select_all()

    def insert_reference(self, internal_id, wordpress_id, scopus_id):
        self.reference_model.insert(internal_id, wordpress_id, scopus_id)

    def insert_comment_reference(self, internal_id, wordpress_post_id, wordpress_comment_id, scopus_id):
        """
        Inserts a new comment reference into the comment reference database

        :param internal_id: The internally created id for the post
        :param wordpress_post_id: The id of the wordpress post, the comment is added to
        :param wordpress_comment_id: The comment id specifically
        :param scopus_id: The scopus id of the publication on which the comment is based on
        :return: void
        """
        self.comment_reference_model.insert(
            internal_id,
            wordpress_post_id,
            wordpress_comment_id,
            scopus_id
        )

    def select_comment_reference_list_py_post(self, wordpress_post_id):
        """
        Gets a list of all the comment references for the post specified by the wordpress id

        :param wordpress_post_id: The int id of the wordpress post
        :return: [(internal id, post id, comment id, scopus id)]
        """
        return self.comment_reference_model.select_by_wordpress_post_id(wordpress_post_id)

    def insert_publication(self, publication, wordpress_id, scopus_id):
        assert isinstance(publication, Publication)

        self.reference_model.insert(publication.id, wordpress_id, scopus_id)

    def publication_from_scopus(self, scopus_publication):
        # Getting an id from the id manager
        publication_id = self.id_manager.new()

        # Creating the publication object
        publication = Publication.from_scopus_publication(scopus_publication, publication_id)
        return publication

    def select_post_reference_by_scopus(self, scopus_id):
        """
        Select the reference by the scopus id.

        :param scopus_id: The int scopus id
        :return: (internal id, wordpress id, scopus id)
        """
        return self.reference_model.search_by_scopus(scopus_id)

    def select_post_reference_by_wordpress(self, wordpress_post_id):
        """
        Select the reference by the wordpress id of the post.

        :param wordpress_post_id: The int wordpress id
        :return: (internal id, wordpress id, scopus id)
        """
        return self.reference_model.search_by_wordpress(wordpress_post_id)

    def wipe(self):
        self.reference_model.wipe()
        self.comment_reference_model.wipe()
        self.id_manager.used_ids = []
        self.id_manager.pointer = 1

    def save(self):
        self.reference_model.save()

    def close(self):
        self.reference_model.save()
        self.id_manager.save()


# TODO: Think about dependency injection for database access?

class PostReferenceModel:

    def __init__(self):
        self.id_manager = IDManagerSingleton.get_instance()  # type: IDManagerSingleton

        # The actual data base access
        self.database_access = MySQLDatabaseAccess()

        self.config = Config.get_instance()
        self.database_name = self.config['MYSQL']['post_reference_table']

    def wipe(self):
        sql = (
            'TRUNCATE TABLE {database};'
            'COMMIT;'
        ).format(
            database=self.database_name
        )
        self.database_access.execute(sql)

    def update_comments_updated_datetime(self, scopus_id, datetime_object):
        # TODO: Make general and scopus id usable
        sql = (
            'UPDATE {database} '
            'SET '
            'comments_updated_datetime = {comments_updated} '
            'WHERE '
            'scopus_id = {scopus_id}'
        ).format(
            database=self.database_name,
            comments_updated=datetime_object.strftime(DATETIME_FORMAT),
            scopus_id=scopus_id
        )

        self.database_access.execute(sql)

    def save(self):
        self.database_access.save()

    def select_all(self):
        """
        Returns a list of all reference tuples, currently saved in the reference database. A reference tuple contains
        the internal id as the first item, the wordpress id as the second, the scopus id as the third.

        :return: A list of tuples with three items
        """
        sql = (
            'SELECT '
            'internal_id, '
            'wordpress_id,'
            'scopus_id,'
            'comments_updated_datetime '
            'FROM {database} '
        ).format(
            database=self.database_name
        )
        row_list = self.database_access.select(sql)
        return row_list

    def select(self, internal_id):
        sql = (
            'SELECT '
            'internal_id, '
            'wordpress_id,'
            'scopus_id,'
            'comments_updated_datetime '
            'FROM {database} '
            'WHERE id={internal_id}'
        ).format(
            database=self.database_name,
            internal_id=internal_id
        )

        row_list = self.database_access.select(sql)
        return row_list[0]
        # TODO: an if decision if actually exists and possibly exception

    def insert(self, internal_id, wordpress_id, scopus_id):
        sql = (
            'INSERT INTO {database}'
            '('
            'internal_id, '
            'wordpress_id, '
            'scopus_id '
            ')'
            'VALUES'
            '('
            '{internal_id},'
            '{wordpress_id}, '
            '{scopus_id}'
            ') '
            'ON DUPLICATE KEY UPDATE '
            'internal_id = {internal_id},'
            'wordpress_id = {wordpress_id},'
            'scopus_id = {scopus_id};'
            'COMMIT;'
        ).format(
            database=self.database_name,
            internal_id=internal_id,
            wordpress_id=wordpress_id,
            scopus_id=scopus_id
        )

        self.database_access.execute(sql)

    def search_by_wordpress(self, wordpress_id):
        sql = (
            'SELECT '
            'internal_id, '
            'wordpress_id, '
            'scopus_id, '
            'comments_updated_datetime '
            'FROM {database} '
            'WHERE '
            'wordpress_id={wordpress_id}'
        ).format(
            database=self.database_name,
            wordpress_id=wordpress_id
        )

        row_list = self.database_access.select(sql)
        return row_list[0]

    def search_by_scopus(self, scopus_id):
        """
        Searches a reference entry by the scopus id and returns a tuple with the internal id, worpdress id and the
        scopus id

        :param scopus_id: The int scopus id for the post
        :return: (internal id, wordpress id, scopus id)
        """
        sql = (
            'SELECT '
            'internal_id,'
            'wordpress_id,'
            'scopus_id,'
            'comments_updated_datetime '
            'FROM {database} '
            'WHERE '
            'scopus_id={scopus_id}'
        ).format(
            database=self.database_name,
            scopus_id=scopus_id
        )

        row_list = self.database_access.select(sql)
        return row_list[0]


class CommentReferenceModel:

    def __init__(self):

        self.config = Config.get_instance()
        # Getting the database table name for the comment reference table
        self.database_name = self.config['MYSQL']['comment_reference_table']

        self.database_access = MySQLDatabaseAccess()

    def insert(self, internal_id, wordpress_post_id, wordpress_citation_id, scopus_id):
        """
        Inserts a new reference entry into the system.

        :param internal_id: The internal id of the ScopusWp program
        :param wordpress_post_id: The int id of the wordpress post, on which the comment was posted
        :param wordpress_citation_id: The int wordpress id of the comment
        :param scopus_id: The scopus id of the publication, which was posted as the comment
        :return: void
        """
        sql = (
            'INSERT INTO {database}'
            '('
            'internal_id,'
            'wordpress_post_id,'
            'wordpress_comment_id,'
            'scopus_id'
            ')'
            'VALUES'
            '('
            '{internal_id},'
            '{wordpress_post_id},'
            '{wordpress_citation_id},'
            '{scopus_id}'
            ')'
            'ON DUPLICATE KEY UPDATE '
            'internal_id = {internal_id},'
            'wordpress_post_id = {wordpress_post_id},'
            'wordpress_comment_id = {wordpress_citation_id},'
            'scopus_id = {scopus_id};'
            'COMMIT;'
        ).format(
            database=self.database_name,
            internal_id=internal_id,
            wordpress_post_id=wordpress_post_id,
            wordpress_citation_id=wordpress_citation_id,
            scopus_id=scopus_id
        )

        self.database_access.execute(sql)

    def select(self, internal_id):
        """
        A reference tuple selected by its internal id.

        :param internal_id: The int internal id
        :return: (internal id, wordpress post id, wordpress comment id, scopus id)
        """
        sql = (
            'SELECT '
            'internal_id,'
            'wordpress_post_id,'
            'wordpress_comment_id,'
            'scopus_id '
            'FROM {database} '
            'WHERE internal_id={internal_id};'
        ).format(
            database=self.database_name,
            internal_id=internal_id
        )

        row_list = self.database_access.select(sql)

        if len(row_list) != 1:
            # TODO: Think of whether to throw an error or w/e
            pass

        return row_list[0]

    def select_all(self):
        """
        A list with all the reference tuple objects.

        :return: [(internal id, wordpress post id, wordpress comment id, scopus id)]
        """
        sql = (
            'SELECT '
            'internal_id,'
            'wordpress_post_id,'
            'wordpress_comment_id,'
            'scopus_id '
            'FROM {database} '
        ).format(
            database=self.database_name
        )

        row_list = self.database_access.select(sql)

        return row_list

    def select_by_wordpress_post_id(self, wordpress_post_id):
        """
        Gets a specific comment reference tuple, by the wordpress id of the post, on which the comment was posted to.

        :param wordpress_post_id: The int id of the wordpress post.
        :return: (internal id, wordpress post id, wordpress comment id, scopus id)
        """
        sql = (
            'SELECT '
            'internal_id,'
            'wordpress_post_id,'
            'wordpress_comment_id,'
            'scopus_id '
            'FROM {database} '
            'WHERE '
            'wordpress_post_id={wordpress_post_id}'
        ).format(
            database=self.database_name,
            wordpress_post_id=wordpress_post_id
        )

        row_list = self.database_access.select(sql)

        return row_list

    def wipe(self):
        """
        Completely deletes all the entries in the database.

        :return: void
        """
        sql = (
            'TRUNCATE {database};'
            'COMMIT;'
        ).format(
            database=self.database_name
        )

        self.database_access.execute(sql)



from ScopusWp.config import PATH

from ScopusWp.scopus.data import ScopusAuthorObservation

import os
import json
import configparser

#############
#  CLASSES  #
#############


class AuthorObservationInterface:

    def get_observation(self, author):
        raise NotImplementedError()

    def contains_author(self, author):
        raise NotImplementedError()

    def values(self):
        raise NotImplementedError()

    def keys(self):
        raise NotImplementedError()

    def __getitem__(self, item):
        raise NotImplementedError()

    def __contains__(self, item):
        raise NotImplementedError()

###############
# Controllers #
###############


class ScopusObservationController:

    def __init__(self):

        self.author_observation_model = AuthorObservationModel()

    def get_author_keywords(self, author):
        author_observation = self.author_observation_model[int(author)]  # type: ScopusAuthorObservation
        return author_observation.keywords

    def all_observations(self):
        return self.author_observation_model.values()

    def all_observed_ids(self):
        return self.author_observation_model.keys()

    def get_publication_keywords(self, publication):
        keywords = []
        author_observation_id_set = set(map(lambda x: int(x), self.author_observation_model.keys()))
        publication_author_id_set = set(map(lambda x: int(x), publication.authors))

        publication_observed_authors = list(publication_author_id_set.intersection(author_observation_id_set))
        # Going through all the authors of the publication and adding all the keywords of each observed author found
        for author in publication_observed_authors:
            author_keywords = self.get_author_keywords(author)
            difference = list(set(author_keywords) - set(keywords))
            keywords += difference
        return keywords

    def filter(self, publication_list):
        whitelist_publications = []
        blacklist_publications = []
        remaining_publications = []

        for publication in publication_list:
            _is_blacklist = False
            _is_whitelist = False
            # Checking the affiliations for all the authors
            for author in publication.authors:
                author_id = int(author.id)
                if author_id in self.author_observation_model.keys():
                    author_observation = self.author_observation_model[author_id] # type: ScopusAuthorObservation
                    _is_whitelist = author_observation.whitelist_contains_any(author.affiliations)
                    # If there was a blacklist found it would be overwritten with the next author, like this a TRUE
                    # will be preserved until the end of the loop
                    if not _is_blacklist:
                        _is_blacklist = author_observation.blacklist_contains_any(author.affiliations)

                    # In the case of whitelist, the publication will be added instantly, but for blacklist it is
                    # decided after all authors have been iterated
                    if _is_whitelist:
                        whitelist_publications.append(publication)
                        break

            if _is_blacklist:
                blacklist_publications.append(publication)
            else:
                remaining_publications.append(publication)

        return whitelist_publications, blacklist_publications, remaining_publications



############
#  Models  #
############


class AuthorObservationModel:

    def __init__(self):
        self.path_string = '{}/scopus/authors.ini'.format(PATH)
        self.config = configparser.ConfigParser()
        self.config.read(self.path_string)

        self.dict, self.author_observations = self.load()

    def __contains__(self, item):
        self.contains_author(item)

    def __getitem__(self, item):
        return self.get_observation(item)

    def get_observation(self, author):
        return self.dict[int(author)]

    def contains_author(self, author):

        return int(author) in self.keys()

    def keys(self):
        return list(self.dict.keys())

    def values(self):
        return self.author_observations

    def load(self):
        content_dict = {}
        author_observation_list = []
        for section in self.config.keys():
            if section == 'DEFAULT':
                continue

            sub_dict = dict(self.config[section])

            # Getting the author observation
            author_observation = self._get_author_observation(sub_dict)
            author_observation_list.append(author_observation)
            # Adding a reference to the same author observation to all the author ids in the list
            for author_id in author_observation.ids:
                content_dict[int(author_id)] = author_observation

        return content_dict, author_observation_list

    @staticmethod
    def _get_author_observation(sub_dict):

        scopus_ids_json_string = sub_dict['ids']
        scopus_ids = json.loads(scopus_ids_json_string)

        keywords_json_string = sub_dict['keywords']
        keywords = json.loads(keywords_json_string)

        whitelist_json_string = sub_dict['scopus_whitelist']
        whitelist = json.loads(whitelist_json_string)

        blacklist_json_string = sub_dict['scopus_blacklist']
        blacklist = json.loads(blacklist_json_string)

        author_observation = ScopusAuthorObservation(
            scopus_ids,
            sub_dict['first_name'],
            sub_dict['last_name'],
            keywords,
            whitelist,
            blacklist
        )

        return author_observation


class ObservedAuthorsModel2:

    def __init__(self):
        self.path = os.path.join(PATH, 'authors2.ini')

        self.source = configparser.ConfigParser()
        self.source.read(self.path)

        self.author_observations = []
        self.author_ids = []

        self.author_observation_dict = {}
        # TODO: eventually dict struture for categories and ids to observ. objects

        for key in self.source.keys():
            if key == 'DEFAULT':
                continue
            # Each key represents one observed author
            value_dict = dict(self.source[key])

            author_observation = self._get_author_observation(value_dict)
            author_id_list = author_observation.ids
            self.author_ids += author_id_list
            self.author_observations.append(author_observation)
            for author_id in author_id_list:
                self.author_observation_dict[author_id] = author_observation

    def filter(self, publication_list):
        whitelist_publications = []
        blacklist_publications = []
        remaining_publications = []

        for publication in publication_list:
            _is_blacklist = False
            _is_whitelist = False
            # Checking the affiliations for all the authors
            for author in publication.authors:
                author_id = int(author.id)
                if author_id in self.author_ids:
                    author_observation = self.author_observation_dict[author_id]
                    _is_whitelist = author_observation.check_whitelist(author.affiliations)
                    # If there was a blacklist found it would be overwritten with the next author, like this a TRUE
                    # will be preserved until the end of the loop
                    if not _is_blacklist:
                        _is_blacklist = author_observation.check_blacklist(author.affiliations)

                    # In the case of whitelist, the publication will be added instantly, but for blacklist it is
                    # decided after all authors have been iterated
                    if _is_whitelist:
                        whitelist_publications.append(publication)
                        break

            if _is_blacklist:
                blacklist_publications.append(publication)
            else:
                remaining_publications.append(publication)

        return whitelist_publications, blacklist_publications, remaining_publications

    def get_observed_authors(self):
        return self.author_ids

    def _get_author_observation(self, value_dict):
        (
            first_name,
            last_name,
            author_id_list,
            keyword_list,
            scopus_whitelist,
            scopus_blacklist
        ) = self._extract_value_dict(value_dict)

        # Creating a new AuthorObservation object
        author_observation = AuthorObservation(
            author_id_list,
            first_name,
            last_name,
            keyword_list,
            scopus_whitelist,
            scopus_blacklist
        )

        return author_observation

    def _extract_value_dict(self, value_dict):
        first_name = value_dict['first_name']
        last_name = value_dict['last_name']

        author_ids_json = value_dict['ids']
        author_id_list = json.loads(author_ids_json)

        scopus_whitelist_json = value_dict['scopus_whitelist']
        scopus_whitelist = json.loads(scopus_whitelist_json)

        scopus_blacklist_json = value_dict['scopus_blacklist']
        scopus_blacklist = json.loads(scopus_blacklist_json)

        keyword_list_json = value_dict['keywords']
        keyword_list = json.loads(keyword_list_json)

        return first_name, last_name, author_id_list, keyword_list, scopus_whitelist, scopus_blacklist

    def __contains__(self, item):
        """
        The magic method for the 'xx in xx' Operator. If given an Author / AuthorProfile object or a author id string
        or int, checks and returns whether that author described by the author id is in the set of observed author ids

        :param item: Author/AuthorProfile/string/int anything that describes an author
        :return: The boolean value of whether or not that author is with the observed authors
        """
        return int(item) in self.author_ids

    def __getitem__(self, item):
        """
        Returns the AuthorObservation object for the given author

        :param item: Author/AuthorProfile/string/int anything that describes an author
        :return: The AuthorObservation object for the given author
        """
        # Works with the author id given directly as string/int and with the Author/AuthorProfile objects
        author_id = int(item)
        return self.author_observation_dict[author_id]  # type: AuthorObservation